package com.cours.dao;

import java.util.List;

import javax.persistence.*;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.cours.exception.GestionPersonnesException;
import com.cours.utils.Constants;
import com.cours.entities.Personne;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

@Repository("personneDao")
public class PersonneDao implements IPersonneDao {
	private static final Log log = LogFactory.getLog(PersonneDao.class);
	@PersistenceContext
	private EntityManager em;

	@Override
	public List<Personne> findAll() {
		List<Personne> persons = null;
		try {
			// persons = em.createQuery("select person from Personne person",
			// Personne.class).getResultList();
			persons = em.createNamedQuery("Personne.findAll", Personne.class).getResultList();
		} catch (Exception e) {
			log.error("Erreur lors de l'execution de la methode, Exception : " + e);
			throw new GestionPersonnesException("Une erreur s'est produite lors de la recupration des personnes.", 1);
		}
		log.debug("persons : " + persons);
		return persons;
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public Personne create(Personne person) {
		if (person != null) {
			try {
				em.persist(person);
			} catch (Exception e) {
				log.error("Erreur lors de l'execution de la methode, Exception : " + e);
				throw new GestionPersonnesException(
						"Une erreur s'est produite lors de la cration de la personne : [id = " + person.getIdPersonne()
								+ " , prenom = " + person.getPrenom() + " , nom = " + person.getNom() + "]",
						2);
			}
		}
		return person;
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public Personne update(Personne person) {
		if (person != null) {
			try {
				person = em.merge(person);
			} catch (Exception e) {
				log.error("Erreur lors de l'execution de la methode, Exception : " + e);
				throw new GestionPersonnesException(
						"Une erreur s'est produite lors de la mise  jour de la personne : [id = "
								+ person.getIdPersonne() + " , prenom = " + person.getPrenom() + " , nom = "
								+ person.getNom() + "]",
						3);
			}
		}
		return person;
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public boolean delete(Personne person) {
		boolean isOk = false;
		if (person != null) {
			try {
				person = em.find(Personne.class, person.getIdPersonne());
				em.remove(em.merge(person));
				isOk = em.find(Personne.class, person.getIdPersonne()) == null;
			} catch (Exception e) {
				log.error("Erreur lors de l'execution de la methode, Exception : " + e);
				throw new GestionPersonnesException(
						"Une erreur s'est produite lors de la suppression de la personne : [id = "
								+ person.getIdPersonne() + " , prenom = " + person.getPrenom() + " , nom = "
								+ person.getNom() + "]",
						4);
			}
		}
		return isOk;
	}

	@Override
	public Personne findById(Integer id) {
		Personne person = null;
		try {
			person = em.find(Personne.class, id);
		} catch (Exception e) {
			log.error("Erreur lors de l'execution de la methode, Exception : " + e);
			throw new GestionPersonnesException(
					"Une erreur s'est produite lors de la recupration de la personne [id = " + person.getIdPersonne()
							+ "]",
					5);
		}
		return person;
	}

	@Override
	public List<Personne> findByPrenom(String prenom) {
		List<Personne> personnes = null;
		try {
			personnes = em
					.createQuery("select person from Personne person where person.prenom=:firstName", Personne.class)
					.setParameter("firstName", prenom).getResultList();
		} catch (Exception e) {
			log.error("Erreur lors de l'execution de la methode, Exception : " + e);
			throw new GestionPersonnesException(
					"Une erreur s'est produite lors de la recupration des personnes [prenom = " + prenom + "]", 6);
		}
		return personnes;
	}

	@Override
	public List<Personne> findByNom(String nom) {
		List<Personne> personnes = null;
		try {
			personnes = em.createQuery("select person from Personne person where person.nom=:lastName", Personne.class)
					.setParameter("lastName", nom).getResultList();
		} catch (Exception e) {
			log.error("Erreur lors de l'execution de la methode, Exception : " + e);
			throw new GestionPersonnesException(
					"Une erreur s'est produite lors de la recupration des personnes [nom = " + nom + "]", 7);
		}
		return personnes;
	}

	@Override
	public List<Personne> findByPrenomNom(String prenom, String nom) {
		List<Personne> personnes = null;
		try {
			personnes = em.createQuery(
					"select person from Personne person where  person.prenom=:firstName and person.nom=:lastName",
					Personne.class).setParameter("firstName", prenom).setParameter("lastName", nom).getResultList();
		} catch (Exception e) {
			log.error("Erreur lors de l'execution de la methode, Exception : " + e);
			throw new GestionPersonnesException(
					"Une erreur s'est produite lors de la recupration des personnes [nom = " + nom + "]", 8);
		}
		return personnes;
	}

	@Override
	public List<Personne> findByCodePostal(String codePostal) {
		List<Personne> personnes = null;
		try {
			personnes = em
					.createQuery("select person from Personne person where person.codePostal=:zipCode", Personne.class)
					.setParameter("zipCode", codePostal).getResultList();
		} catch (Exception e) {
			log.error("Erreur lors de l'execution de la methode, Exception : " + e);
			throw new GestionPersonnesException(
					"Une erreur s'est produite lors de la recupration des personnes [codePostal = " + codePostal + "]",
					9);
		}
		return personnes;
	}

	@Override
	public List<Personne> findByVille(String ville) {
		List<Personne> personnes = null;
		try {
			personnes = em.createQuery("select person from Personne person where person.ville=:town", Personne.class)
					.setParameter("town", ville).getResultList();
		} catch (Exception e) {
			log.error("Erreur lors de l'execution de la methode, Exception : " + e);
			throw new GestionPersonnesException(
					"Une erreur s'est produite lors de la recupration des personnes [ville = " + ville + "]", 10);
		}
		return personnes;
	}

	@Override
	public List<Personne> findPersonnesObese() {
		List<Personne> persons = null;
		try {
			persons = em.createQuery(
					"select person from Personne person where  person.poids/(person.taille/100 * person.taille/100) >:paramObese",
					Personne.class).setParameter("paramObese", Constants.LIMITE_SUP_SURPOIDS).getResultList();
		} catch (Exception e) {
			log.error("Erreur lors de l'execution de la methode, Exception : " + e);
			throw new GestionPersonnesException("Une erreur s'est produite lors de la recupration des personnes.", 1);
		}
		log.debug("persons : " + persons);
		return persons;
	}

	@Override
	public List<Personne> findPersonnesMaigre() {
		List<Personne> persons = null;
		try {
			persons = em.createQuery(
					"select person from Personne person where  person.poids/(person.taille/100 * person.taille/100) <:paramMaigre",
					Personne.class).setParameter("paramMaigre", Constants.LIMITE_INF_MAIGRE).getResultList();
		} catch (Exception e) {
			log.error("Erreur lors de l'execution de la methode, Exception : " + e);
			throw new GestionPersonnesException("Une erreur s'est produite lors de la recupration des personnes.", 1);
		}
		log.debug("persons : " + persons);
		return persons;
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public List<Personne> createBatchPersonnes(List<Personne> persons) {
		boolean isOk = false;
		int nbPersonneCreated = 0;
		log.debug("1 : persons : " + persons);
		if (persons != null && !persons.isEmpty()) {
			for (Personne person : persons) {
				try {
					em.persist(person);
					nbPersonneCreated++;
				} catch (Exception e) {
					log.error("Erreur lors de l'execution de la methode, Exception : " + e);
					throw new GestionPersonnesException(
							"Une erreur s'est produite lors de la cration en masse des personnes, Exception pour la personne : [id = "
									+ person.getIdPersonne() + " , prenom = " + person.getPrenom() + " , nom = "
									+ person.getNom() + "]",
							6);
				}
			}
			log.debug("2 : persons : " + persons);
			if (nbPersonneCreated != persons.size()) {
				persons = null;
			}
		}
		log.debug("isOk : " + isOk + " , persons : " + persons);
		return persons;
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public Boolean deleteBatchPersonnes(List<Personne> persons) {
		boolean isOk = false;
		int nbPersonneDeleted = 0;
		log.debug("1 : persons : " + persons);
		if (persons != null && !persons.isEmpty()) {
			for (Personne person : persons) {
				try {
					person = em.find(Personne.class, person.getIdPersonne());
					em.remove(em.merge(person));
					nbPersonneDeleted++;
				} catch (Exception e) {
					log.error("Erreur lors de l'execution de la methode, Exception : " + e);
					throw new GestionPersonnesException(
							"Une erreur s'est produite lors de la suppresion en masse des personnes, Exception pour la personne : [id = "
									+ person.getIdPersonne() + " , prenom = " + person.getPrenom() + " , nom = "
									+ person.getNom() + "]",
							7);
				}
			}
			log.debug("2 : personnes : " + persons);
			if (nbPersonneDeleted == persons.size()) {
				isOk = true;
			}
		}
		log.debug("isOk : " + isOk + " , persons : " + persons);
		return isOk;
	}
}